@@ -6,8 +6,11 @@
 @ GitHub : https://github.com/ZeroSeeker
 @ Gitee : https://gitee.com/ZeroSeeker
 """
+from .basics import calculate_sha256_doc
+from .basics import convert_decimals
 from .basics import Basics
 import showlog
+import copy
 import time
 import envx
 default_silence = False
@@ -84,7 +87,8 @@ def safe_find(
         connect_str: str = None,
         env_file_name: str = default_env_file_name,
         silence: bool = default_silence,
-        show_id: bool = True
+        show_id: bool = True,
+        distinct_by: str = None
 ):
     """
     查询
@@ -99,6 +103,7 @@ def safe_find(
     :param env_file_name:
     :param silence:
     :param show_id: 是否返回id
+    :param distinct_by: 取某字段的唯一值
     """
     # ---------------- 固定设置 ----------------
     if connect_str is None:
@@ -126,7 +131,8 @@ def safe_find(
                 show_setting=show_setting,
                 sort_setting=sort_setting,
                 limit_num=limit_num,
-                skip_num=skip_num
+                skip_num=skip_num,
+                distinct_by=distinct_by
             )
             return response
         except ConnectionError:
@@ -192,9 +198,9 @@ def safe_find_page(
         db: str = None,
         collection: str = None,
         query: dict = None,
-        previous_tag: str = None,
+        previous_key: str = None,
         previous_value: str = None,
-        where_str: str = '$gt',
+        # where_str: str = '$gt',
         show_setting: dict = None,
         sort_setting: list = None,
         limit_num: int = 10,
@@ -208,7 +214,7 @@ def safe_find_page(
     :param db:
     :param collection:
     :param query:
-    :param previous_tag:
+    :param previous_key:
     :param previous_value:
     :param where_str:
     :param show_setting:
@@ -240,9 +246,9 @@ def safe_find_page(
                 db=db,
                 collection=collection,
                 query=query,
-                previous_tag=previous_tag,
+                previous_key=previous_key,
                 previous_value=previous_value,
-                where_str=where_str,
+                # where_str=where_str,
                 show_setting=show_setting,
                 sort_setting=sort_setting,
                 limit_num=limit_num
@@ -381,12 +387,19 @@ def safe_insert(
         collection: str = None,
         connect_str: str = None,  # 若指定，将优先使用
         env_file_name: str = default_env_file_name,
-        silence: bool = default_silence
+        silence: bool = default_silence,
+        index_list: list = None,
+        add_sha256: bool = False
 ):
     # ---------------- 固定设置 ----------------
     if connect_str is None:
         connect_str = make_con_info(env_file_name=env_file_name)
     # ---------------- 固定设置 ----------------
+
+    if add_sha256:
+        for value in values:
+            value["_sha256"] = calculate_sha256_doc(doc=value)
+
     while True:
         try:
             mongo_basics = Basics(
@@ -396,9 +409,10 @@ def safe_insert(
                 silence=silence,
             )
             response = mongo_basics.insert(
-                values=values,
+                values=convert_decimals(copy.deepcopy(values)),
                 db=db,
-                collection=collection
+                collection=collection,
+                index_list=index_list
             )
             return response
         except ConnectionError:
@@ -418,12 +432,27 @@ def safe_upsert(
         query_keys: list = None,
         connect_str: str = None,  # 若指定，将优先使用
         env_file_name: str = default_env_file_name,
-        silence: bool = default_silence
+        silence: bool = default_silence,
+        index_list: list = None,
+        add_sha256: bool = False
 ):
     # ---------------- 固定设置 ----------------
     if connect_str is None:
         connect_str = make_con_info(env_file_name=env_file_name)
     # ---------------- 固定设置 ----------------
+
+    if add_sha256:
+        for value in values:
+            value["_sha256"] = calculate_sha256_doc(doc=value)
+
+    if index_list:
+        index_list_new = copy.deepcopy(index_list)
+    else:
+        index_list_new = list()
+
+    if query_keys:
+        index_list_new.extend(query_keys)
+
     while True:
         try:
             mongo_basics = Basics(
@@ -433,10 +462,11 @@ def safe_upsert(
                 silence=silence,
             )
             response = mongo_basics.upsert(
-                values=values,
+                values=convert_decimals(copy.deepcopy(values)),
                 db=db,
                 collection=collection,
-                query_keys=query_keys
+                query_keys=query_keys,
+                index_list=index_list_new
             )
             return response
         except ConnectionError:
@@ -525,3 +555,71 @@ def safe_delete_many(
             if silence is False:
                 showlog.error('未知错误')
             time.sleep(1)
+
+
+def collections(
+        db: str = None,
+        connect_str: str = None,  # 若指定，将优先使用
+        env_file_name: str = default_env_file_name,
+        silence: bool = default_silence
+):
+    """
+    获取所有的集合名
+    """
+    # ---------------- 固定设置 ----------------
+    if connect_str is None:
+        connect_str = make_con_info(env_file_name=env_file_name)
+    # ---------------- 固定设置 ----------------
+    while True:
+        try:
+            mongo_basics = Basics(
+                connect_str=connect_str,
+                db=db,
+                silence=silence,
+            )
+            return mongo_basics.collections(
+                db=db,
+            )
+        except ConnectionError:
+            if silence is False:
+                showlog.warning('连接失败，将重试...')
+            time.sleep(1)
+        except Exception as ex:
+            if silence is False:
+                showlog.error('未知错误')
+            time.sleep(1)
+
+
+def drop_collection(
+        db: str,
+        collection: str,
+        connect_str: str = None,  # 若指定，将优先使用
+        env_file_name: str = default_env_file_name,
+        silence: bool = default_silence
+):
+    """
+    删除集合
+    """
+    # ---------------- 固定设置 ----------------
+    if connect_str is None:
+        connect_str = make_con_info(env_file_name=env_file_name)
+    # ---------------- 固定设置 ----------------
+    while True:
+        try:
+            mongo_basics = Basics(
+                connect_str=connect_str,
+                db=db,
+                silence=silence,
+            )
+            return mongo_basics.drop_collection(
+                db=db,
+                collection=collection,
+            )
+        except ConnectionError:
+            if silence is False:
+                showlog.warning('连接失败，将重试...')
+            time.sleep(1)
+        except Exception as ex:
+            if silence is False:
+                showlog.error('未知错误')
+            time.sleep(1)
